#include "neslib.h"
#include "global.h"

#include "actor.h"
#include "hero.h"
#include "ai.h"
#include "map.h"

void __fastcall__ actor_updatePosition(u8 i);
void __fastcall__ actor_updateOam(u8 i);

static u8 a_id[MAX_ACTORS];
static u8 a_tx[MAX_ACTORS];
static u8 a_ty[MAX_ACTORS];
static u8 a_x[MAX_ACTORS];
static u8 a_y[MAX_ACTORS];
static u8 a_xFrac[MAX_ACTORS];
static u8 a_yFrac[MAX_ACTORS];
static s8 a_vx[MAX_ACTORS];
static s8 a_vy[MAX_ACTORS];
static u8 a_vxFrac[MAX_ACTORS];
static u8 a_vyFrac[MAX_ACTORS];
static u8 a_gfxPtr[MAX_ACTORS];
static u8 a_flags[MAX_ACTORS];
static u8 a_dispY[MAX_ACTORS];
static u8 a_timer[MAX_ACTORS];
static u8 a_anim[MAX_ACTORS];
static u8 a_animPtr[MAX_ACTORS];
static u8 a_animDelay[MAX_ACTORS];
static u8 a_speed[MAX_ACTORS];
static u8 a_moveTimer[MAX_ACTORS];
static u8 a_state[MAX_ACTORS];

static u8 aiTmp0;
static u8 aiTmp1;

const s8 DIR_DX[] = {-1, 0, 1, 0};
const s8 DIR_DY[] = { 0,-1, 0, 1};
const u16 SPEED[] = {0x1000/64,0x1000/60,0x1000/56,0x1000/52,0x1000/48,0x1000/44,0x1000/40,0x1000/36,0x1000/32,0x1000/28,0x1000/24,0x1000/20,0x1000/16,0x1000/12,0x1000/8,0x1000/4};
const u16 SPEED_TIMER[] = {64, 60, 56, 52, 48, 44, 40, 36, 32, 28, 24, 20, 16, 12, 8, 4};

const u8 GD_BUNNY[] = {0x02,0x03,0x12,0x13, 0x22,0x23,0x32,0x33, 0x42,0x43,0x52,0x53, 0x62,0x63,0x72,0x73, 0x82,0x83,0x92,0x93};
const u8 GD_SHADOW[] = {0x14,0x15,0x00,0x00};
const u8 GD_ENEMY[] = {0x06,0x07,0x16,0x17, 0x26,0x27,0x36,0x37};
const u8 GD_ENEMY2[] = {0x08,0x09,0x18,0x19, 0x28,0x29,0x38,0x39};
const u8 GD_ENEMY3[] = {0x0A,0x0B,0x1A,0x1B, 0x2A,0x2B,0x3A,0x3B};
const u8 GD_ENEMY4[] = {0x0C,0x0D,0x1C,0x1D, 0x2C,0x2D,0x3C,0x3D};
const u8 GD_CAT[] = {0x0E,0x0F,0x1E,0x1F, 0x2E,0x2F,0x3E,0x3F, 0x4E,0x4F,0x5E,0x5F};

extern s8 ANIM_BUNNY[];
extern s8 ANIM_SHADOW[];
extern s8 ANIM_ENEMY[];
extern s8 ANIM_ENEMY2[];
extern s8 ANIM_ENEMY3[];
extern s8 ANIM_ENEMY4[];
extern s8 ANIM_CAT[];

#define SIZE_8x8 0
#define SIZE_16x16 1
#define SIZE_16x8 2
//null,bunny,shadow,enemy1,enemy2,enemy3,enemy4,cat
const u8 AD_SIZE[] = 							{0, 		SIZE_16x16,	SIZE_16x8,	SIZE_16x16,	SIZE_16x16,	SIZE_16x16,	SIZE_16x16,	SIZE_16x16,	};
const u8 *AD_GFXDATA[] =						{0, 		GD_BUNNY, 	GD_SHADOW,	GD_ENEMY,	GD_ENEMY2,	GD_ENEMY3,	GD_ENEMY4,	GD_CAT,		};
const s8 *AD_ANIMDATA[] =						{0, 		ANIM_BUNNY, ANIM_SHADOW,ANIM_ENEMY,	ANIM_ENEMY2,ANIM_ENEMY3,ANIM_ENEMY4,ANIM_CAT,	};
const u8 AD_PAL[] =								{0,			0,			0,			0,			1,			2,			3,			0,			};
const void __fastcall__ (*AD_AIFUNC[])(u8,u8) =	{ai_none, 	ai_bunny,	ai_shadow,	ai_enemy,	ai_enemy2,	ai_enemy3,	ai_enemy4,	ai_cat,		};

static u8 oamPtr;


void __fastcall__ actor_destroy(u8 i) {
	a_id[i] = 0;
}

void __fastcall__ actor_replace(u8 i, u8 id) {
	a_id[i] = id;
	a_flags[i] &= ~0x03;
	a_flags[i] |= AD_PAL[id];
}

void __fastcall__ actor_createAt(u8 i, u8 id, u8 tx, u8 ty) {
	a_id[i] = id;
	a_tx[i] = tx;
	a_x[i] = tx*16+8;
	a_ty[i] = ty;
	a_y[i] = ty*16+8;
	a_xFrac[i] = 0;
	a_yFrac[i] = 0;
	a_vx[i] = 0;
	a_vy[i] = 0;
	a_vxFrac[i] = 0;
	a_vyFrac[i] = 0;
	a_flags[i] = AD_PAL[id];
	a_dispY[i] = 0;
	a_timer[i] = 0;
	a_gfxPtr[i] = 0;
	a_animPtr[i] = 0;
	a_animDelay[i] = 0;
	a_speed[i] = 0;
	a_state[i] = 0;
	a_anim[i] = 0xFF;
	a_moveTimer[i] = 0xFF;
	ai_init(i);
}

u8 __fastcall__ actor_create(u8 id, u8 tx, u8 ty) {
	static u8 i;
	for(i=0; i<ENEMY_INDEX_START; ++i) {
		if(a_id[i] == 0) {
			actor_createAt(i, id, tx, ty);
			return i;
		}
	}
}

u8 __fastcall__ actor_createEnemy(u8 id, u8 tx, u8 ty) {
	static u8 i;
	for(i=ENEMY_INDEX_START; i<MAX_ACTORS; ++i) {
		if(a_id[i] == 0) {
			actor_createAt(i, id, tx, ty);
			return i;
		}
	}
}

u8 __fastcall__ actor_move(u8 i, u8 dir) {
	static u8 t;
	static s8 dx,dy;
	static u16 v, speed;
	if((a_x[i]&15) == 8 && (a_y[i]&15) == 8) {
		dx = DIR_DX[dir];
		dy = DIR_DY[dir];
		t = getTileIndex(a_tx[i]+dx, a_ty[i]+dy);
		
		__asm__("lda %v",t);
		__asm__("tay");
		__asm__("lda _map,y");
		__asm__("tay");
		__asm__("lda _tileFlags,y");
		__asm__("sta %v",t);
		
		if(!(t & TILEFLAG_SOLID)) {
			speed = SPEED[a_speed[i]];
			v = speed * dx;
			if(dx == 0xFF)
				v = speed * -1;
			else
				v = speed * dx;
			a_vx[i] = v>>8;
			a_vxFrac[i] = v&0xFF;
			v = speed * dy;
			if(dy == 0xFF)
				v = speed * -1;
			else
				v = speed * dy;
			a_vy[i] = v>>8;
			a_vyFrac[i] = v&0xFF;
			
			a_moveTimer[i] = SPEED_TIMER[a_speed[i]];
			return 1;
		}
	}
	return 0;
}


void __fastcall__ actor_callAI(u8 i, u8 sf) {
	AD_AIFUNC[a_id[i]](i, sf);
}

/*void __fastcall__ actor_updateOam(u8 i) {
	static u8 *gfxData;
	static u8 id;
	static u8 gfxPtr;
	static u8 x, y;
	if(a_flags[i] & ACTORFLAG_INVISIBLE)
		return;
	id = a_id[i];
	gfxData = (u8*)AD_GFXDATA[id];
	gfxPtr = a_gfxPtr[i];
	switch(AD_SIZE[id]) {
		case SIZE_8x8:
			oamPtr = oam_spr(a_x[i]-4, a_y[i]-4+a_dispY[i], gfxData[a_gfxPtr[i]], a_flags[i], oamPtr);
			break;
		case SIZE_16x16:
			x = a_x[i];
			y = a_y[i] + a_dispY[i];
			oamPtr = oam_spr(x-8, y-8, gfxData[gfxPtr++], a_flags[i], oamPtr);
			oamPtr = oam_spr(x, y-8, gfxData[gfxPtr++], a_flags[i], oamPtr);
			oamPtr = oam_spr(x-8, y, gfxData[gfxPtr++], a_flags[i], oamPtr);
			oamPtr = oam_spr(x, y, gfxData[gfxPtr], a_flags[i], oamPtr);
			break;
		case SIZE_16x8:
			x = a_x[i];
			y = a_y[i] + a_dispY[i];
			oamPtr = oam_spr(x-8, y-4, gfxData[gfxPtr++], a_flags[i], oamPtr);
			oamPtr = oam_spr(x, y-4, gfxData[gfxPtr++], a_flags[i], oamPtr);
			break;
	}
}*/

void __fastcall__ actor_doCollisions(u8 i) {
	for(i=ENEMY_INDEX_START; i<MAX_ACTORS; ++i) {
		if(heroTX == a_tx[i] && heroTY == a_ty[i]) {
			ai_collision(0,i);
			//ai_collision(i,0);
			break; //1 collision per frame, avoid slowdown
		}
	}
}

void __fastcall__ actor_updateAll(void) {
	static u8 i;
	oamPtr = 0;
	for(i=0; i<MAX_ACTORS; ++i) {
		if(a_id[i] != 0) {
			actor_updatePosition(i);
			actor_doCollisions(i);
			
			if(i!=0) { //not hero?
				if(a_moveTimer[i]!=0xFF) {
					--a_moveTimer[i];
					if(a_moveTimer[i] == 0) {
						a_moveTimer[i] = 0xFF;
						a_xFrac[i] = 0;
						a_yFrac[i] = 0;
						a_vx[i] = 0;
						a_vy[i] = 0;
						a_vxFrac[i] = 0;
						a_vyFrac[i] = 0;
					}
				}
				//hero is handled seperately in hero.c
				a_tx[i] = (a_x[i])>>MAP_WSHIFT;
				a_ty[i] = (a_y[i])>>MAP_WSHIFT;
			}
			
			ai_update(i);
			if(a_id[i]==0) //actor destroyed itself in ai_update?
				break;
			++a_timer[i];
			/*if(i==1) {
				MEM(0xFC) = a_anim[i];
				MEM(0xFD) = a_animDelay[i];
				MEM(0xFE) = a_animPtr[i];
				MEM(0xFF) = a_gfxPtr[i];
			}*/
			
			actor_updateAnimation(i);
			actor_updateOam(i);
			
		}
	}
	
	oam_hide_rest(oamPtr);
}






