#include "neslib.h"
#include "global.h"

#include "actor.h"
#include "hero.h"
#include "ai.h"
#include "map.h"
#include "game.h"

void __fastcall__ ai_none(u8, u8 sf) {
	switch(sf) {
		case AI_INIT:
			break;
		case AI_UPDATE:
			break;
		case AI_COLLISION: //aiTmp0(in:2nd actor)
			break;
	}
}

void __fastcall__ ai_bunny(u8 a, u8 sf) {
	switch(sf) {
		case AI_INIT:
			break;
		case AI_UPDATE:
			if(heroState == HEROSTATE_DYING && a_timer[a] == 60) {
				actor_destroy(a);
				loadNewLevel = currentLevel+1;
			}
			break;
		case AI_COLLISION: //aiTmp0(in:2nd actor)
			if(heroState != HEROSTATE_DYING && a_state[aiTmp0]==ENEMYSTATE_NORMAL) {
				heroState = HEROSTATE_DYING;
				actor_setAnimation(a, AN_BUNNY_DEATH);
				a_timer[a] = 0;
				game_playSound(SOUND_DEATH);
			}
			break;
	}
}

void __fastcall__ ai_cat(u8, u8 sf) {
	switch(sf) {
		case AI_INIT:
			break;
		case AI_UPDATE:
			break;
		case AI_COLLISION: //aiTmp0(in:2nd actor)
			if(a_state[aiTmp0] == ENEMYSTATE_NORMAL) {
				a_state[aiTmp0] = ENEMYSTATE_DYING;
				a_vx[aiTmp0] = DIR_DX[heroMoveDir] * 3;
				a_vy[aiTmp0] = DIR_DY[heroMoveDir] * 3;
				a_vxFrac[aiTmp0] = 0;
				a_vyFrac[aiTmp0] = 0;
				a_moveTimer[aiTmp0] = 0xFF;
				game_playSound(SOUND_PUSH_ENEMY);
			}
			break;
	}
}

void __fastcall__ ai_shadow(u8 a, u8 sf) {
	switch(sf) {
		case AI_INIT:
			actor_setAnimation(a, AN_SHADOW_IDLE);
			break;
		case AI_UPDATE:
			if(heroMoveTimer==0 || heroId!=ACTORID_BUNNY || heroState==HEROSTATE_DYING) {
				a_flags[a] |= ACTORFLAG_INVISIBLE;
			}
			else {
				a_flags[a] &= ~ACTORFLAG_INVISIBLE;
				a_x[a] = heroX;
				a_y[a] = heroY+7;
			}
			break;
	}
}

void __fastcall__ backOff(u8 a) {
	static u8 result;
	actor_setAnimation(a, AN_ENEMY_SCARED);
	if(heroTX > a_tx[a])
		result = actor_move(a, DIR_LEFT);
	else if(heroTX < a_tx[a])
		result = actor_move(a, DIR_RIGHT);
	if(result == 0) {
		if(heroTY >= a_ty[a])
			actor_move(a, DIR_UP);
		else if(heroTY < a_ty[a])
			actor_move(a, DIR_DOWN);
	}
}

void __fastcall__ enemySpawning(u8 a) {
	if((a_timer[a]>>1) & 1)
		a_flags[a] &= ~ACTORFLAG_INVISIBLE;
	else
		a_flags[a] |= ACTORFLAG_INVISIBLE;
	if(a_timer[a] > 80) {
		a_state[a] = ENEMYSTATE_NORMAL;
		a_flags[a] &= ~ACTORFLAG_INVISIBLE;
	}
}

void __fastcall__ enemyDying(u8 a) {
	if(a_x[a]<16 || a_x[a]>256-32
		|| a_y[a]<48 || a_y[a]>240-16) {
		actor_destroy(a);
	}
}

void __fastcall__ ai_enemy(u8 a, u8 sf) {
	static u8 result;
	switch(sf) {
		case AI_INIT:
			a_speed[a] = enemyBaseSpeed+1;
			break;
		case AI_UPDATE:
			switch(a_state[a]) {
				case ENEMYSTATE_NORMAL:
					//prioritize horizontal movement
					result = 0;
					if(heroId == ACTORID_BUNNY) {
						actor_setAnimation(a, AN_ENEMY_IDLE);
						if(heroTX < a_tx[a])
							result = actor_move(a, DIR_LEFT);
						else if(heroTX > a_tx[a])
							result = actor_move(a, DIR_RIGHT);
						if(result == 0) {
							if(heroTY <= a_ty[a])
								actor_move(a, DIR_UP);
							else if(heroTY > a_ty[a])
								actor_move(a, DIR_DOWN);
						}
					}
					else {
						backOff(a);
					}
					break;
				case ENEMYSTATE_DYING:
					enemyDying(a);
					break;
				case ENEMYSTATE_SPAWNING:
					enemySpawning(a);
					break;
			}
			break;
	}
}

void __fastcall__ ai_enemy2(u8 a, u8 sf) {
	static u8 result;
	switch(sf) {
		case AI_INIT:
			a_speed[a] = enemyBaseSpeed;
			break;
		case AI_UPDATE:
			switch(a_state[a]) {
				case ENEMYSTATE_NORMAL:
					//prioritize vertical movement
					result = 0;
					if(heroId == ACTORID_BUNNY) {
						actor_setAnimation(a, AN_ENEMY_IDLE);
						if(heroTY < a_ty[a])
							result = actor_move(a, DIR_UP);
						else if(heroTY > a_ty[a])
							result = actor_move(a, DIR_DOWN);
						if(result == 0) {
							if(heroTX < a_tx[a])
								actor_move(a, DIR_LEFT);
							else if(heroTX >= a_tx[a])
								actor_move(a, DIR_RIGHT);
						}
					}
					else {
						backOff(a);
					}
					break;
				case ENEMYSTATE_DYING:
					enemyDying(a);
					break;
				case ENEMYSTATE_SPAWNING:
					enemySpawning(a);
					break;
			}
			break;
	}
}

void __fastcall__ ai_enemy3(u8 a, u8 sf) {
	static u8 destTX,destTY;
	static u8 result;
	switch(sf) {
		case AI_INIT:
			a_speed[a] = enemyBaseSpeed+1;
			break;
		case AI_UPDATE:
			switch(a_state[a]) {
				case ENEMYSTATE_NORMAL:
					destTX = heroTX + DIR_DX[heroMoveDir]*3;
					destTY = heroTY + DIR_DY[heroMoveDir]*3;
					if(heroId == ACTORID_BUNNY) {
						actor_setAnimation(a, AN_ENEMY_IDLE);
						//prioritize horizontal movement
						if(destTX < a_tx[a])
							result = actor_move(a, DIR_LEFT);
						else if(destTX > a_tx[a])
							result = actor_move(a, DIR_RIGHT);
						if(result == 0) {
							if(destTY <= a_ty[a])
								actor_move(a, DIR_UP);
							else if(destTY > a_ty[a])
								actor_move(a, DIR_DOWN);
						}
					}
					else {
						backOff(a);
					}
					break;
				case ENEMYSTATE_DYING:
					enemyDying(a);
					break;
				case ENEMYSTATE_SPAWNING:
					enemySpawning(a);
					break;
			}
			break;
	}
}

void __fastcall__ ai_enemy4(u8 a, u8 sf) {
	static u8 destTX,destTY;
	static u8 result;
	switch(sf) {
		case AI_INIT:
			a_speed[a] = enemyBaseSpeed;
			break;
		case AI_UPDATE:
			switch(a_state[a]) {
				case ENEMYSTATE_NORMAL:
					destTX = heroTX + DIR_DX[heroMoveDir]*3;
					destTY = heroTY + DIR_DY[heroMoveDir]*3;
					if(heroId == ACTORID_BUNNY) {
						actor_setAnimation(a, AN_ENEMY_IDLE);
						//prioritize vertical movement
						if(destTY < a_ty[a])
							result = actor_move(a, DIR_UP);
						else if(destTY > a_ty[a])
							result = actor_move(a, DIR_DOWN);
						if(result == 0) {
							if(destTX < a_tx[a])
								actor_move(a, DIR_LEFT);
							else if(destTX >= a_tx[a])
								actor_move(a, DIR_RIGHT);
						}
					}
					else {
						backOff(a);
					}
					break;
				case ENEMYSTATE_DYING:
					enemyDying(a);
					break;
				case ENEMYSTATE_SPAWNING:
					enemySpawning(a);
					break;
			}
			break;
	}
}
