#include "neslib.h"
#include "global.h"
#include "game.h"

#include "actor.h"
#include "map.h"
#include "hero.h"
#include "ai.h"

static u8 frameCount;
static u8 seconds;
static u8 pad1;
static u8 pad1Trigger;
static u8 loadNewLevel;

void __fastcall__ game_playSound(u8 n) {
	static u8 i;
	switch(n) {
		case SOUND_PUSH_ENEMY:
			MEM(0x4000) = 0x5F;
			__asm__("lda #$59");
			__asm__("sta $4002");
			__asm__("lda #$01");
			__asm__("sta $4003");
			break;
		case SOUND_CARROT:
			//__asm__("lda #%01011111");
			MEM(0x4000) = 0x5F;
			__asm__("lda #$C9");
			__asm__("sta $4002");
			__asm__("lda #$00");
			__asm__("sta $4003");
			break;
		case SOUND_DEATH:
			for(i=0; i<10; ++i) {
				ppu_waitnmi();
				MEM(0x4000) = 0x5F;
				__asm__("lda #$F9");
				__asm__("sta $4002");
				__asm__("lda #$00");
				__asm__("sta $4003");
				ppu_waitnmi();
				MEM(0x4000) = 0x5F;
				__asm__("lda #$59");
				__asm__("sta $4002");
				__asm__("lda #$01");
				__asm__("sta $4003");
				ppu_waitnmi();
				MEM(0x4000) = 0x5F;
				__asm__("lda #$39");
				__asm__("sta $4002");
				__asm__("lda #$01");
				__asm__("sta $4003");
			}
			break;
		case SOUND_ADVANCE:
			for(i=0; i<2; ++i) {
				ppu_waitnmi();
				MEM(0x4000) = 0x5F;
				__asm__("lda #$C9");
				__asm__("sta $4002");
				__asm__("lda #$00");
				__asm__("sta $4003");
				ppu_waitnmi();
				ppu_waitnmi();
				ppu_waitnmi();
				MEM(0x4000) = 0x5F;
				__asm__("lda #$49");
				__asm__("sta $4002");
				__asm__("lda #$00");
				__asm__("sta $4003");
				ppu_waitnmi();
				ppu_waitnmi();
			}
			break;
	}
}


void __fastcall__ game_init(void) {
	actor_create(ACTORID_SHADOW, 0, 0);
}

void __fastcall__ game_switchHero(void) {
	//static u8 i;
	/*for(i=ENEMY_INDEX_START; i<MAX_ACTORS; ++i) {
		//reset to base speed
		a_speed[i] = enemyBaseSpeed;
	}*/
	if(heroId == ACTORID_BUNNY) {
		actor_replace(0, ACTORID_CAT);
		MEM(PAL_BUF+18) = 0x20;
		MEM(PAL_BUF+22) = 0x20;
		MEM(PAL_BUF+26) = 0x20;
		MEM(PAL_BUF+30) = 0x20;
		
		MEM(PAL_BUF+19) = 0x25;
		seconds = 3;
	}
	else {
		actor_replace(0, ACTORID_BUNNY);
		MEM(PAL_BUF+18) = 0x25;
		MEM(PAL_BUF+22) = 0x21;
		MEM(PAL_BUF+26) = 0x29;
		MEM(PAL_BUF+30) = 0x27;
		
		MEM(PAL_BUF+19) = 0x20;
		seconds = 7;
	}
}

void __fastcall__ changeHeroPalette(u8 a) {
	if(heroId == ACTORID_CAT)
		a = (a+1)&1;
	if(a==0)
		MEM(PAL_BUF+19) = 0x25;
	else 
		MEM(PAL_BUF+19) = 0x20;
}


void __fastcall__ game_update(void) {
	static u8 i;
	pad1Trigger = pad_trigger(0);
	pad1 = pad_state(0);
	
	if(loadNewLevel != 0) {
		ppu_off();
		map_load(loadNewLevel-1);
		loadNewLevel = 0;
		ppu_on_all();
		ppu_waitnmi();
	}
	
	
	//hero switching
	if(heroState != HEROSTATE_DYING) {
		if(++frameCount==60) {
			/*for(i=ENEMY_INDEX_START; i<MAX_ACTORS; ++i) {
				++a_speed[i];
			}*/
			frameCount = 0;
			if(--seconds == 0) {
				game_switchHero();
			}
		}
		if(seconds==2) {
			if(frameCount>30 && (frameCount&0x07) == 0) {
				changeHeroPalette((frameCount>>3)&1);
			}
		}
		else if(seconds==1) {
			if((frameCount&0x03) == 0) {
				changeHeroPalette((frameCount>>2)&1);
			}
		}
	}
	
	//enemy spawning
	for(i=0; i<MAX_ENEMIES; ++i) {
		if(enemyId[i]!=0 && a_id[ENEMY_INDEX_START+i] == 0) {
			if(--enemyRespawnTimer[i] == 0) {
				actor_createAt(ENEMY_INDEX_START+i, enemyId[i], enemySpawnX[i], enemySpawnY[i]);
				enemyRespawnTimer[i] = 120;
				a_state[ENEMY_INDEX_START+i] = ENEMYSTATE_SPAWNING;
			}
		}
	}
	
	hero_update();
	
	actor_updateAll();
}




