#include "neslib.h"
#include "global.h"
#include "hero.h"

#include "actor.h"
#include "map.h"
#include "game.h"

static u8 heroMoveTimer;
static u8 heroMoveTimerMax;
static s8 heroMoveTimerInc;
static u8 heroMoveDir;
static u8 moveDX;
static u8 moveDY;
static u8 oppositeDir;

const u8 BUNNY_JUMP_DISPY[] = {0,-1,-2,-3,-4,-4,-5,-5,-5,-5,-4,-4,-3,-2,-1,0};
//const u8 BUNNY_JUMP_DISPY[] = {0,-1,-2,-3,-4,-5,-6,-7,-8,-7,-6,-5,-4,-3,-2,-1};

void __fastcall__ hero_create(u8 tx, u8 ty) {
	actor_create(ACTORID_BUNNY, tx, ty);
	heroState = HEROSTATE_NORMAL;
	heroY -= 4;
	heroMoveTimer = 0;
}


u8 __fastcall__ hero_move(s8 dx, s8 dy) {	
	static u8 t;
	t = getTileIndex(heroTX+dx, heroTY+dy);
	
	__asm__("lda %v",t);
	__asm__("tay");
	__asm__("lda _map,y");
	__asm__("sta %v",t);
	
	if(!(tileFlags[t] & TILEFLAG_SOLID)
		|| (t == TILE_POST && heroId == ACTORID_BUNNY)) {
		heroVX = dx;
		moveDX = dx;
		heroVY = dy;
		moveDY = dy;
		heroMoveTimerInc = -1;
		if(heroId == ACTORID_BUNNY) {
			heroMoveTimer = 16;
			heroMoveTimerMax = 16;
			
			heroAnim = 0; //force animation reset
			//a_animPtr[0] = 0;
			//a_animDelay[0] = 1;
			//actor_setAnimation(0, AN_BUNNY_JUMP);
		}
		else {
			heroVX *= 2;
			heroVY *= 2;
			heroMoveTimer = 8;
			heroMoveTimerMax = 8;
			//a_animPtr[0] = 0; //force animation reset
			//a_animDelay[0] = 1;
			//actor_setAnimation(0, AN_CAT_RUN);
		}
		return 1;
	}
	return 0;
}

void __fastcall__ hero_update(void) {
	//no hero logic when hero is dying
	if(heroState == HEROSTATE_DYING) {
		heroVX = 0;
		heroVY = 0;
		return;
	}
	
	if(heroMoveTimer!=0) {
		heroMoveTimer += heroMoveTimerInc;
		if(heroMoveTimer == 0 || heroMoveTimer==heroMoveTimerMax) {
			heroMoveTimer = 0;
			heroVX = 0;
			heroVY = 0;
		}
		if(heroId == ACTORID_BUNNY)
			heroDispY = BUNNY_JUMP_DISPY[heroMoveTimer];
		else
			heroDispY = 0;
	}
	
	//update tile position when hero is halfway between two tiles
	heroTX = (heroX)>>MAP_WSHIFT;
	heroTY = (heroY+4)>>MAP_WSHIFT;
	
	if(heroMoveTimer==0 && getTile(heroTX,heroTY) == TILE_POST)
		map_setAndUpdateTile(heroTX, heroTY, TILE_FLATPOST);
		
	/*if(heroMoveTimerInc==0xFF) {
		if(heroMoveTimer == 7) {
			heroTX += moveDX;
			heroTY += moveDY;
			if(getTile(heroTX,heroTY) == TILE_POST)
				map_setAndUpdateTile(heroTX, heroTY, TILE_FLATPOST);
		}
	}
	else {
		if(heroMoveTimer == 8) {
			heroTX -= moveDX;
			heroTY -= moveDY;
			if(getTile(heroTX,heroTY) == TILE_POST)
				map_setAndUpdateTile(heroTX, heroTY, TILE_FLATPOST);
		}
	}*/
	
	if(heroMoveTimer == 0) {
		if(pad1 & PAD_LEFT) {
			if(hero_move(-1,0)) {
				oppositeDir = PAD_RIGHT;
				heroMoveDir = DIR_LEFT;
			}
		}
		else if(pad1 & PAD_RIGHT) {
			if(hero_move(1,0)) {
				oppositeDir = PAD_LEFT;
				heroMoveDir = DIR_RIGHT;
			}
		}
		else if(pad1 & PAD_UP) {
			if(hero_move(0,-1)) {
				oppositeDir = PAD_DOWN;
				heroMoveDir = DIR_UP;
			}
		}
		else if(pad1 & PAD_DOWN) {
			if(hero_move(0,1)) {
				oppositeDir = PAD_UP;
				heroMoveDir = DIR_DOWN;
			}
		}
	}
	else {
		//allow player to return to previous tile while still moving
		if(pad1 & oppositeDir) {
			heroMoveTimerInc = -heroMoveTimerInc;
			heroVX = -heroVX;
			heroVY = -heroVY;
			if(oppositeDir == PAD_LEFT)
				oppositeDir = PAD_RIGHT;
			else if(oppositeDir == PAD_RIGHT)
				oppositeDir = PAD_LEFT;
			else if(oppositeDir == PAD_UP)
				oppositeDir = PAD_DOWN;
			else if(oppositeDir == PAD_DOWN)
				oppositeDir = PAD_UP;
		}
	}
	
	//collect carrots
	if(getTile(heroTX, heroTY) == TILE_CARROT) {
		map_setAndUpdateTile(heroTX, heroTY, 1);
		--carrotCount;
		
		if(carrotCount==0) {
			game_playSound(SOUND_ADVANCE);
			++currentLevel;
			loadNewLevel = currentLevel+1;
		}
		else
			game_playSound(SOUND_CARROT);
	}
	
}



