#include "neslib.h"
#include "global.h"

#include "map.h"
#include "actor.h"
#include "hero.h"
#include "game.h"

extern const u8 PALETTE[32];

const u8 tiles0[] =		{0x00,			0x02,	0x04,				0x06,	0x08,				0x0A};
const u8 tiles1[] =		{0x01,			0x03,	0x05,				0x07,	0x09,				0x0B};
const u8 tiles2[] =		{0x10,			0x12,	0x14,				0x16,	0x18,				0x1A};
const u8 tiles3[] =		{0x11,			0x13,	0x15,				0x17,	0x19,				0x1B};
const u8 tileFlags[] = 	{TILEFLAG_SOLID,	1,	0|TILEFLAG_SOLID,	1,		0|TILEFLAG_SOLID,	0};


static u8 map[MAP_SIZE];
static u8 carrotCount;
static u8 attribute;
static u8 currentLevel;

static u8 enemyId[MAX_ENEMIES];
static u8 enemySpawnX[MAX_ENEMIES];
static u8 enemySpawnY[MAX_ENEMIES];
static u8 enemyRespawnTimer[MAX_ENEMIES];
static u8 enemyBaseSpeed;

void __fastcall__ map_setAndUpdateTile(u8 x, u8 y, u8 t) {
	//no attribute updates, yet!
	static u16 addr;
	static u8 buf[3*4];
	static u8 i;
	setTile(x, y, t);
	i = 0;
	addr = x*2 + y*64 + 0x2000;
	buf[i++] = addr>>8;
	buf[i++] = addr&0xFF;
	buf[i++] = tiles0[t];
	++addr;
	buf[i++] = addr>>8;
	buf[i++] = addr&0xFF;
	buf[i++] = tiles1[t];
	addr += 31;
	buf[i++] = addr>>8;
	buf[i++] = addr&0xFF;
	buf[i++] = tiles2[t];
	++addr;
	buf[i++] = addr>>8;
	buf[i++] = addr&0xFF;
	buf[i] = tiles3[t];
	set_vram_update(sizeof(buf), buf);
}

void __fastcall__ makeAttribute(u8 tile) {
	attribute <<= 2;
	attribute |= tileFlags[tile]&TILEFLAG_PALETTE;
}


const char MAP_LEVEL01[] = {
"xxxxxxxxxxxxxxxx"
"xxxxxxxxxxxxxxxx"
"x      @      xx"
"x w w w w w w xx"
"x             xx"
"x w w w w w w xx"
"x    c   c    xx"
"x w w w w w w xx"
"x    c   c    xx"
"x w w w w w w xx"
"x             xx"
"x w w w w w w xx"
"xc 2       4 cxx"
"xxxxxxxxxxxxxxxx"
"xxxxxxxxxxxxxxxx"
};
const char MAP_LEVEL02[] = {
"xxxxxxxxxxxxxxxx"
"xxxxxxxxxxxxxxxx"
"xc     @     cxx"
"x w w w w w w xx"
"x             xx"
"x w w w w w w xx"
"xc     c     cxx"
"x w w w1w w w xx"
"xc     c     cxx"
"x w w w w w w xx"
"x             xx"
"x w w w w w w xx"
"x  2       3  xx"
"xxxxxxxxxxxxxxxx"
"xxxxxxxxxxxxxxxx"
};
const char MAP_LEVEL03[] = {
"xxxxxxxxxxxxxxxx"
"xxxxxxxxxxxxxxxx"
"x      @      xx"
"x wcw w w wcw xx"
"x ccc     ccc xx"
"x wcw w w wcw xx"
"x             xx"
"x wcw w w wcw xx"
"x ccc     ccc xx"
"x wcw w2w wcw xx"
"x     1 3     xx"
"x w w w4w w w xx"
"x             xx"
"xxxxxxxxxxxxxxxx"
"xxxxxxxxxxxxxxxx"
};
const char MAP_LEVEL04[] = {
"xxxxxxxxxxxxxxxx"
"xxxxxxxxxxxxxxxx"
"xxxxxxxxxxxxxxxx"
"x w w w@w w w xx"
"x             xx"
"x w w w w w w xx"
"x             xx"
"x wcw w w wcw xx"
"x  c       c  xx"
"x wcw w2w wcw xx"
"xccccc1 3cccccxx"
"x w w w4w w w xx"
"xxxxxxxxxxxxxxxx"
"xxxxxxxxxxxxxxxx"
"xxxxxxxxxxxxxxxx"
};
const char MAP_LEVEL05[] = {
"xxxxxxxxxxxxxxxx"
"xxxxxxxxxxxxxxxx"
"x      @      xx"
"x w w w w w w xx"
"x  c       c  xx"
"x w w w w w w xx"
"x             xx"
"x w wcwcwcw w xx"
"x  c       c  xx"
"x wcw w2w wcw xx"
"x  c  143  c  xx"
"x wcwcwcwcwcw xx"
"x             xx"
"xxxxxxxxxxxxxxxx"
"xxxxxxxxxxxxxxxx"
};
const char MAP_LEVEL06[] = {
"xxxxxxxxxxxxxxxx"
"xxxxxxxxxxxxxxxx"
"x1     @     3xx"
"x w w w w w w xx"
"x  c       c  xx"
"x w wcwcwcw w xx"
"x             xx"
"x w wcwcwcw w xx"
"x             xx"
"x w wcwcwcw w xx"
"x  c       c  xx"
"x w w w w w w xx"
"x4     c     2xx"
"xxxxxxxxxxxxxxxx"
"xxxxxxxxxxxxxxxx"
};
const char MAP_LEVEL07[] = {
"xxxxxxxxxxxxxxxx"
"xxxxxxxxxxxxxxxx"
"x     w@w     xx"
"x     www     xx"
"x      c      xx"
"x             xx"
"x www w  w ww xx"
"x w   ww w w wxx"
"x www w ww w wxx"
"x w   w  w w wxx"
"x www w  w ww xx"
"x             xx"
"x             xx"
"xxxxxxxxxxxxxxxx"
"xxxxxxxxxxxxxxxx"
};

const char MAP_LEVEL08[] = {
"xxxxxxxxxxxxxxxx"
"xxxxxxxxxxxxxxxx"
"x     w@w     xx"
"x     www     xx"
"x             xx"
"x             xx"
"x  w   wwww   xx"
"x      w   w  xx"
"x      w   w  xx"
"x      wwww   xx"
"x  w   w      xx"
"x      w      xx"
"x             xx"
"xxxxxxxxxxxxxxxx"
"xxxxxxxxxxxxxxxx"
};

/*
const char MAP_LEVEL01[] = {
"xxxxxxxxxxxxxxxx"
"xxxxxxxxxxxxxxxx"
"x      @      xx"
"x w w w w w w xx"
"x             xx"
"x wow w w wow xx"
"x  o c   c o  xx"
"x wow w1w wow xx"
"x  o c   c o  xx"
"x wow w w wow xx"
"x             xx"
"xowow w w wowoxx"
"xc 1       3 cxx"
"xxxxxxxxxxxxxxxx"
"xxxxxxxxxxxxxxxx"
};

const char MAP_LEVEL02[] = {
"xxxxxxxxxxxxxxxx"
"xxxxxxxxxxxxxxxx"
"x             xx"
"x w w w w w w xx"
"x             xx"
"x wowowwwowow xx"
"x  oc4o o1co  xx"
"x wow w@w wow xx"
"x  oc2o o3co  xx"
"x wowowwwowow xx"
"x             xx"
"x w w w w w w xx"
"x     ccc     xx"
"xxxxxxxxxxxxxxxx"
"xxxxxxxxxxxxxxxx"
};

const char MAP_LEVEL03[] = {
"xxxxxxxxxxxxxxxx"
"xxxxxxxxxxxxxxxx"
"x    o @ o    xx"
"x w w wow w w xx"
"x             xx"
"x wowow wowow xx"
"x  o c   c o  xx"
"x wow w1w wow xx"
"x  o c   c o  xx"
"x wowowowowow xx"
"x             xx"
"xowow w w wowoxx"
"xc 2o     o3 cxx"
"xxxxxxxxxxxxxxxx"
"xxxxxxxxxxxxxxxx"
};
*/

void __fastcall__ resetEnemies(void) {
	static u8 i;
	for(i=0; i<MAX_ENEMIES; ++i) {
		enemyId[i] = 0;
	}
	for(i=ENEMY_INDEX_START; i<MAX_ACTORS; ++i) {
		actor_destroy(i);
	}
}

void __fastcall__ addEnemy(u8 id, u8 spawnX, u8 spawnY) {
	static u8 i;
	for(i=0; i<MAX_ENEMIES; ++i) {
		if(enemyId[i]==0) {
			enemyId[i] = id;
			enemySpawnX[i] = spawnX;
			enemySpawnY[i] = spawnY;
			enemyRespawnTimer[i] = 60;
			return;
		}
	}
}


void __fastcall__ map_load(u8 level) {
	static u8 i,x,y,t;
	static u8 startX, startY;
	static const u8* mapData;
	const u8 *levels[] = 		{MAP_LEVEL01,MAP_LEVEL02,MAP_LEVEL03,MAP_LEVEL04,MAP_LEVEL05,MAP_LEVEL06,MAP_LEVEL07,MAP_LEVEL08};
	const u8 levelBaseSpeed[] = {          8,          8,          8,         9,           9,         10,          1,          1};
	
	carrotCount = 0;
	
	seconds = 7;
	frameCount = 0;
	currentLevel = level;
	enemyBaseSpeed = currentLevel + 6;
	mapData = levels[currentLevel];
	
	resetEnemies();
	memfill(map, 0, MAP_SIZE);
	
	i = 0;
	for(y=0; y<MAP_HEIGHT; ++y) {
		for(x=0; x<MAP_WIDTH; ++x) {
			map[i] = TILE_FLOOR;
			switch(mapData[i]) {
				case 'x': map[i] = TILE_NONE; break;
				case 'w': map[i] = TILE_WALL; break;
				case 'o': map[i] = TILE_POST; break;
				case 'c': map[i] = TILE_CARROT; ++carrotCount; break;
				case '@': startX = x; startY = y; break;
				case '1': addEnemy(ACTORID_ENEMY,x,y); break;
				case '2': addEnemy(ACTORID_ENEMY2,x,y); break;
				case '3': addEnemy(ACTORID_ENEMY3,x,y); break;
				case '4': addEnemy(ACTORID_ENEMY4,x,y); break;
			}
			++i;
		}			
	}
	
	/*for(i=0; i<sizeof(PALETTE); ++i) {
		pal_col(i,PALETTE[i]);
	}*/
	pal_all(PALETTE);
	
	//tiles
	vram_adr(0x2000);
	for(y=0; y<MAP_HEIGHT; ++y) {
		//even row
		for(i=0; i<MAP_WIDTH; ++i) {
			t = getTile(i, y);
			vram_put(tiles0[t]);
			vram_put(tiles1[t]);
		}
		//odd row
		for(i=0; i<MAP_WIDTH; ++i) {
			t = getTile(i, y);
			vram_put(tiles2[t]);
			vram_put(tiles3[t]);
		}
	}
	
	//attributes
	vram_adr(0x23C0);
	for(y=0; y<MAP_HEIGHT; y+=2) {
		for(i=0; i<MAP_WIDTH; i+=2) {
			t = getTileIndex(i, y);
			makeAttribute(map[t+0x11]);
			makeAttribute(map[t+0x10]);
			makeAttribute(map[t+1]);
			makeAttribute(map[t]);
			vram_put(attribute);
		}
	}
	
	heroId = 0;
	hero_create(startX, startY);
	/*actor_createEnemy(ACTORID_ENEMY, 3, 7);
	actor_createEnemy(ACTORID_ENEMY2, 4, 7);
	actor_createEnemy(ACTORID_ENEMY3, 5, 7);
	actor_createEnemy(ACTORID_ENEMY4, 6, 7);*/
}
